local _,L = ...
local rematch = Rematch
local card = RematchPetCard
local settings, roster
local abilityCard = RematchAbilityCard

card.statButtons = {} -- frame pool of RematchPetCardStatTemplate buttons

rematch:InitModule(function()
	rematch.PetCard = card
	settings = RematchSettings
	roster = rematch.Roster
	local font,size,flag = card.Title.Name:GetFont()
	card.Title.Name:SetFont(font,size+2,flag)
	font,size,flag = abilityCard.Name:GetFont()
	abilityCard.Name:SetFont(font,size+2,flag)
	abilityCard.Hints.StrongVs:SetText(L["Vs"])
	abilityCard.Hints.WeakVs:SetText(L["Vs"])
	card.Back.Bottom.DamageTaken:SetText(L["Damage\nTaken"])
	card.Back.Bottom.StrongFrom:SetText(L["from"])
	card.Back.Bottom.WeakFrom:SetText(L["from"])
	card.Back.Bottom.StrongAbilities:SetText(L["abilities"])
	card.Back.Bottom.WeakAbilities:SetText(L["abilities"])
	card.PinButton.tooltipTitle = L["Unpin Pet Card"]
	card.PinButton.tooltipBody = L["While pinned, pet cards will display where you last moved it.\n\nClick this to unpin the pet card and snap it back to the pets."]
	card.PetCardTitle:SetText(L["Pet Card"])
	card.Front.Middle.AltFlipHelp:SetText(L["Hold [Alt] to view more about this pet."])
end)

function rematch:ShowPetCard(parent,petID,force)

	if (card.locked and not force) or rematch:UIJustChanged() then
		return -- don't show a new pet card if current one is locked or just left a menu/frame shown/etc
	end

	if settings.ClickPetCard and not force then
		return
	end

	-- if FastPetCard not enabled, then cause a 0.25 delay before showing a card (unless it's forced)
	if not settings.ClickPetCard and not settings.FastPetCard then
		if parent and petID and not force then
			card.parent = parent
			card.petID = petID
			rematch:StartTimer("PetCard",0.25,rematch.ShowPetCard)
			return
		elseif not force then
			parent = card.parent
			petID = card.petID
		end
	end
	
	if not parent or not petID then return end

	-- search hits are shown only for cards displayed from the pet panel
	-- check if card's parent is a descendant of the pet panel
	local forPetPanel
	local candidate = parent
	repeat
		candidate = candidate:GetParent()
		if not candidate or candidate==UIParent then
			forPetPanel = false
		elseif candidate==rematch.PetPanel then
			forPetPanel = true
		end
	until forPetPanel~=nil

	local leveling,speciesID,customName,level,xp,maxXP,displayID,isFavorite,name,icon,petType,_,sourceText,description,canBattle,isTradable,unique,obtainable

	local idType = rematch:GetIDType(petID)
	if idType=="pet" then
		speciesID,customName,level,xp,maxXP,displayID,isFavorite,name,icon,petType,_,sourceText,description,_,canBattle,isTradable,unique,obtainable = C_PetJournal.GetPetInfoByPetID(petID)
	elseif idType=="species" then
		speciesID = petID
		name,icon,petType,_,sourceText,description,_,canBattle,isTradable,unique,obtainable,displayID = C_PetJournal.GetPetInfoBySpeciesID(petID)
	elseif idType=="leveling" then
		leveling = true
		name = L["Leveling Pet"]
		icon = "Interface\\AddOns\\Rematch\\Textures\\leveling.blp"
	end

	if not speciesID and not leveling then
		return
	end

	-- hide the search hits
	card.Title.Icon.SearchHit:Hide()
	card.Title.Type.SearchHit:Hide()
	for i=1,6 do
		card.Front.Bottom.Abilities[i].SearchHit:Hide()
	end

	-- title stuff
	card.Title.Name:SetText(customName or name)
	if leveling then
		-- for some reason leveling icon doesn't work with mask; going back to SetPortraitTexture for these
		SetPortraitToTexture(card.Title.Icon.Texture,icon)
		card.Title.Type.Texture:SetTexCoord(0,1,0,1)
		SetPortraitToTexture(card.Title.Type.Texture,"Interface\\Icons\\INV_Pet_Achievement_CatchPetFamily25")
	else
		card.Title.Icon.Texture:SetTexture(icon)
		card.Title.Type.Texture:SetTexCoord(0.4921875,0.796875,0.50390625,0.65625)
		card.Title.Type.Texture:SetTexture(petType and "Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
	end

	--[[ Front ]]

	-- filling out middle card info
	local info = card.Front.Middle

	info.Level:SetText(level or "")
	info.LevelBG:SetShown(level and canBattle)
	info.LevelLabel:SetShown(level and canBattle)
	info.Level:SetShown(level and canBattle)

	-- bottom of middle card info
	local ybottom = 6
	-- xp bar
	if canBattle and level and level<25 then
		info.XP:Show()
		info.XP:SetValue(xp/maxXP*100)
		info.XP.Text:SetText(format(L["XP: %d/%d (%d%%)"],xp,maxXP,xp*100/maxXP))
		ybottom = ybottom + 16
	else
		info.XP:Hide()
	end
	-- "Hold [Alt] to flip card etc" help bit
	if not settings.HideMenuHelp then
		info.AltFlipHelp:ClearAllPoints()
		info.AltFlipHelp:SetPoint("BOTTOMLEFT",8,ybottom)
		ybottom = ybottom + info.AltFlipHelp:GetStringHeight()+4
		info.AltFlipHelp:Show()
	else
		info.AltFlipHelp:Hide()
	end
	-- possible breeds
	info.PossibleBreeds:Hide()
	if rematch.breedSource and canBattle and obtainable then
		local breeds = rematch.info
		wipe(breeds)
		if rematch.breedSource=="BattlePetBreedID" then
			if not BPBID_Arrays.BreedsPerSpecies then BPBID_Arrays.InitializeArrays() end
			local data,numBreeds = BPBID_Arrays
			if #rematch.breedNames==0 then
				rematch:GatherBreedNames()
			end
			if data.BreedsPerSpecies[speciesID] then
				for i=1,#data.BreedsPerSpecies[speciesID] do
					local breed = data.BreedsPerSpecies[speciesID][i]
					tinsert(breeds,rematch.breedNames[breed-2])
				end
			end
		elseif rematch.breedSource=="PetTracker_Breeds" then
			if PetTracker.Breeds[speciesID] then
				for _,breed in pairs(PetTracker.Breeds[speciesID]) do
					tinsert(breeds,PetTracker:GetBreedIcon(breed,0.85))
				end
			end
		end
		local possibleBreeds = #breeds>0 and table.concat(breeds,rematch.breedSource=="PetTracker_Breeds" and " " or ", ") or UNKNOWN
		info.PossibleBreeds:SetText(format("%s: \124cffffffff%s",L["Possible Breeds"],possibleBreeds))
		info.PossibleBreeds:ClearAllPoints()
		info.PossibleBreeds:SetPoint("BOTTOMLEFT",8,ybottom)
		ybottom = ybottom + info.PossibleBreeds:GetStringHeight()+4
		info.PossibleBreeds:Show()
	end
	-- collected
	info.Collected:Hide()
	local collected = speciesID and C_PetJournal.GetOwnedBattlePetString(speciesID)
	if collected and canBattle then
		roster:CheckForUpdate() -- make sure roster has info
		local collectedPets = rematch.info
		wipe(collectedPets)
		for _,otherPetID in ipairs(roster.pets) do
			if type(otherPetID)=="string" then
				local otherSpeciesID,_,otherLevel = C_PetJournal.GetPetInfoByPetID(otherPetID)
				if otherSpeciesID==speciesID then
					local _,_,_,_,otherRarity = C_PetJournal.GetPetStats(otherPetID)
					local _,_,_,otherHex = GetItemQualityColor(otherRarity-1)
					if rematch.breedSource then
						tinsert(collectedPets,format("\124c%s%d %s\124r",otherHex,otherLevel,rematch:GetBreed(otherPetID)))
					else
						tinsert(collectedPets,format("\124c%s%s %d\124r",otherHex,LEVEL,otherLevel))
					end
				end
			end
		end
		info.Collected:SetText(format("%s: %s",collected,table.concat(collectedPets,rematch.breedSource=="PetTracker_Breeds" and " " or ", ")))
		info.Collected:ClearAllPoints()
		info.Collected:SetPoint("BOTTOMLEFT",info,"BOTTOMLEFT",8,ybottom)
		ybottom = ybottom + info.Collected:GetStringHeight()+4
		info.Collected:Show()
	end

	local	model = leveling and card.Front.LevelingModel or card.Front.Model
	if leveling then
		model:SetModel("Interface\\Buttons\\talktomequestion_ltblue.m2")
		model:SetModelScale(3)
		card.Front.Model:Hide()
	elseif displayID and displayID~=0 and displayID~=model.displayID then
		model.displayID = displayID
		model:SetDisplayInfo(displayID)
		model:SetDoBlend(false)
		card.Front.LevelingModel:Hide()
	end
	model:SetPoint("BOTTOMRIGHT",info,"BOTTOMRIGHT",-3,ybottom-4)
	model:Show()

	-- stats along left of card
	card.statIndex = 1
	card.ypos = -4

	for _,button in pairs(card.statButtons) do
		button:Hide()
	end

	if customName then
		info.RealName:SetText(name)
		card.ypos = card.ypos - info.RealName:GetStringHeight() - 6
	end
	info.RealName:SetShown(customName and true)

	if idType=="pet" then -- this is a pet player owns

		if C_PetJournal.PetIsSlotted(petID) then
			card:AddStat(L["Slotted"],"Interface\\RaidFrame\\ReadyCheck-Ready",0,1,0,1,L["Slotted"],L["This pet is loaded in one of the three battle pet slots."])
		end

		if isFavorite then
			card:AddStat(L["Favorite"],"Interface\\Common\\FavoritesIcon",0.125,0.71875,0.09375,0.6875,L["Favorite"],L["This pet is marked as a Favorite from its right-click menu."])
		end

		if canBattle then
			if rematch:IsPetLeveling(petID) then
				card:AddStat(L["Leveling"],"Interface\\AddOns\\Rematch\\Textures\\preference",0.25,0.75,0.25,0.75,L["Leveling"],L["This pet is in Rematch's leveling queue."])
			end

			local health,maxHealth,power,speed,rarity = C_PetJournal.GetPetStats(petID)
			local healthText = health==0 and format("%s0/%d (%s)",rematch.hexRed,maxHealth,DEAD) or health<maxHealth and format("%s%d/%d (%d%%)",rematch.hexRed,health,maxHealth,health/maxHealth*100) or maxHealth
			card:AddStat(healthText,"Interface\\PetBattles\\PetBattle-StatIcons",0.5,1,0.5,1,PET_BATTLE_STAT_HEALTH,PET_BATTLE_TOOLTIP_HEALTH_MAX)
			card:AddStat(power,"Interface\\PetBattles\\PetBattle-StatIcons",0,0.5,0,0.5,PET_BATTLE_STAT_POWER,PET_BATTLE_TOOLTIP_POWER)
			card:AddStat(speed,"Interface\\PetBattles\\PetBattle-StatIcons",0,0.5,0.5,1,PET_BATTLE_STAT_SPEED,PET_BATTLE_TOOLTIP_SPEED)

			local r,g,b,hex = GetItemQualityColor(rarity-1)
			card:AddStat(format("\124c%s%s",hex,_G["BATTLE_PET_BREED_QUALITY"..rarity]),"Interface\\PetBattles\\PetBattle-StatIcons",0.5,1,0,0.5,PET_BATTLE_STAT_QUALITY,PET_BATTLE_TOOLTIP_RARITY)
			info.LevelBG:SetVertexColor(r,g,b)

			if rematch.breedSource then
				card:AddStat(rematch:GetBreed(petID),"Interface\\AchievementFrame\\UI-Achievement-Progressive-Shield",0.09375,0.578125,0.140625,0.625,L["Breed"],format(L["Determines how stats are distributed.  All breed data is pulled from your installed %s%s\124r addon."],rematch.hexWhite,GetAddOnMetadata(rematch.breedSource,"Title") or rematch.breedSource))
			end
		end
	end

	if not leveling and roster:IsPetInTeam(petID) then
		card:AddStat(format(L["%d Teams"],settings.Sanctuary[petID][1]),"Interface\\FriendsFrame\\UI-Toast-ChatInviteIcon",0.125,0.875,0.09375,0.84375,L["Teams"],L["The number of Rematch teams that includes this pet."])
	end

	-- abilities
	local abilities = card.Front.Bottom.Abilities
	for i=1,6 do
		abilities[i]:Hide()
	end
	wipe(rematch.abilityList)
	if canBattle then
		C_PetJournal.GetPetAbilityList(speciesID,rematch.abilityList,rematch.levelList)
		for i=1,6 do
			if canBattle and rematch.abilityList[i] then
				local _,abilityName,abilityIcon,_,abilityDescription,_,abilityType,noHints = C_PetBattles.GetAbilityInfoByID(rematch.abilityList[i])
				if abilityName then
					abilities[i].abilityID = rematch.abilityList[i]
					abilities[i].Name:SetText(abilityName)
					abilities[i].Icon:SetTexture(abilityIcon)
					abilities[i].Type:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[abilityType])
					abilities[i]:Show()
					-- show search hits if this is for the pet panel (and if any filters apply)
					if forPetPanel then
						if roster.searchMask and abilityDescription and (abilityName:match(roster.searchMask) or abilityDescription:match(roster.searchMask)) then
							-- if name or description matches search, show its SearchHit silver border
							abilities[i].SearchHit:Show()
						elseif roster:IsFilterUsed("Similar") and roster:GetFilter("Similar",rematch.abilityList[i]) then
							-- if "Similar" filter used and this ability is among the abilities, show its SearchHit
							abilities[i].SearchHit:Show()
						elseif roster:IsFilterUsed("Strong") and not noHints then
							-- show the SearchHit if this ability is Strong Vs the filtered types
							for typeIndex in pairs(settings.Filters.Strong) do
								if rematch.hintsDefense[typeIndex][1] == abilityType then
									abilities[i].SearchHit:Show()
								end
							end
						end
					end

				end
			end
		end
	end
	card.Front.Bottom.CantBattle:SetShown(not canBattle)
	if leveling then
		card.Front.Bottom.CantBattle:SetText(L["When this team loads, your current leveling pet will go in this spot."])
	else
		card.Front.Bottom.CantBattle:SetText(BATTLE_PET_CANNOT_BATTLE)
	end

	-- finally if this is for the pet panel, see if search hit should be shown on title buttons
	if forPetPanel then
		-- show search hit on pet icon if search text is in pet's sourceText
		if roster.searchMask and ((sourceText or ""):match(roster.searchMask) or (name or ""):match(roster.searchMask) or (customName or ""):match(roster.searchMask)) then
			card.Title.Icon.SearchHit:Show()
		end
		-- show search hit on type icon if "Touch vs" or "Types" filter used
		if roster:IsFilterUsed("Tough") or roster:IsFilterUsed("Types") then
			card.Title.Type.SearchHit:Show()
		end
	end

	--[[ Back ]]

	if not leveling then -- leveling pets have no back of card
		-- shrink font if source text is very long (some pets like spiders list nearly every zone in the game!)
		local sourceLength = sourceText:len()
		if sourceLength>300 then
			card.Back.Source.Text:SetFontObject("GameFontHighlightSmall")
			if sourceLength>500 then -- if text is really really long, cut it short
				sourceText = sourceText:sub(1,500).."..."
			end
		else
			card.Back.Source.Text:SetFontObject("GameFontHighlight")
		end
		if not obtainable then
			sourceText = L["This is an opponent pet."]
		end

		card.Back.Bottom.StrongType:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[rematch.hintsDefense[petType][1]])
		card.Back.Bottom.WeakType:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[rematch.hintsDefense[petType][2]])

		card.Back.Source.Text:SetText(format("%s%s%s",sourceText,isTradable and "" or "\n\124cffff0000"..BATTLE_PET_NOT_TRADABLE,unique and "\n\124cffffd200"..ITEM_UNIQUE or ""))
		card.Back.Source:SetHeight(card.Back.Source.Text:GetStringHeight()+16)
		card.Back.Middle.Lore:SetText(description)
		card.Back.Bottom.TypeName:SetText(_G["BATTLE_PET_NAME_"..petType])
		card.Back.Bottom.TypeIcon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
		local racial = select(5,C_PetBattles.GetAbilityInfoByID(PET_BATTLE_PET_TYPE_PASSIVES[petType]))
		racial = racial:match("^.-\r\n(.-)\r"):gsub("%[percentage.-%]%%","4%%")
		card.Back.Bottom.Racial:SetText(racial)
		card.Back.Bottom:SetHeight(104 + card.Back.Bottom.Racial:GetStringHeight())
	end

	--[[ Leftovers ]]

	-- desaturate BGs for missing pets
	if idType=="pet" then
		card.Title.TitleBG:SetDesaturated(false)
		card.Front.Bottom.AbilitiesBG:SetDesaturated(false)
		card.Back.Bottom.BottomBG:SetDesaturated(false)
	else
		card.Title.TitleBG:SetDesaturated(true)
		card.Front.Bottom.AbilitiesBG:SetDesaturated(true)
		card.Back.Bottom.BottomBG:SetDesaturated(true)
	end

	-- adjust abilities for pets with only 3 abilities (unobtainable ones, but not always)
	-- example: Manos/Fatos/Hanos have 3 abilities each; Erris' Sprouts/Runts/Prince Charming have 6
	local bothColumns = #rematch.abilityList>3
	for i=1,3 do
		local abilityOffset = bothColumns and 7 or 54
		local abilityWidth = bothColumns and 114 or 140
		card.Front.Bottom.Abilities[i]:SetPoint("BOTTOMLEFT",abilityOffset,(3-i)*34+7)
		card.Front.Bottom.Abilities[i]:SetWidth(abilityWidth)
	end

	-- position the card
	card:UpdateLockState()
	card:ClearAllPoints()
	if settings.FixedPetCard and settings.PetCardXPos then
		card:SetPoint("CENTER",UIParent,"BOTTOMLEFT",settings.PetCardXPos,settings.PetCardYPos)
		card.PinButton:Show()
	else
		rematch:SmartAnchor(card,parent)
		card.PinButton:Hide()
	end
	card:SetHeight(max(190+abs(card.ypos)+ybottom,416))

	-- save parent and petID to recreate card if needed
	card.parent = parent
	card.petID = petID

	-- show the card
	card:FlipCardIfAltDown()
	rematch:AdjustScale(card)
	card:Show()
end

-- called at end of ShowPetCard and in the OnEvent for MODIFIER_STATE_CHANGED
function card:FlipCardIfAltDown()
	local altDown = IsAltKeyDown() and card.petID~=0 -- don't flip leveling pets
	card.Front:SetShown(not altDown)
	card.Back:SetShown(altDown)
end

-- fills out a stat on the card and returns ypos for next stat
function card:AddStat(text,icon,left,right,top,bottom,title,body)
	if not card.statButtons[card.statIndex] then
		card.statButtons[card.statIndex] = CreateFrame("Frame",nil,card.Front.Middle,"RematchPetCardStatTemplate")
	end
	local button = card.statButtons[card.statIndex]
	button.Text:SetText(text)
	button.Text:SetTextColor(1,1,1)
	button.Icon:SetTexture(icon)
	button.Icon:SetTexCoord(left,right,top,bottom)
	button.tooltipTitle = title
	button.tooltipBody = body
	button:SetPoint("TOPLEFT",8,card.ypos)
	button:Show()
	card.ypos = card.ypos - 20
	card.statIndex = card.statIndex + 1
	return card.statIndex-1
end

-- maybe should be true for OnLeaves
function rematch:HidePetCard(maybe)
	if not settings.FastPetCard then
		rematch:StopTimer("PetCard")
	end
	if maybe and card.locked then return end
	card:Hide()
end

function card:OnShow()
	card:RegisterEvent("MODIFIER_STATE_CHANGED")
end

function card:OnHide()
	card:UnregisterEvent("MODIFIER_STATE_CHANGED")
	card.locked = nil
	card.petID = nil
	rematch.BottomPanel.SummonButton:Disable()
end

function rematch:LockPetCard(parent,petID)
	if petID ~= card.petID then
		if settings.ClickPetCard then
			card.locked = true
		end
		rematch:ShowPetCard(parent,petID,true)
	elseif petID then
		card.locked = not card.locked
		card:UpdateLockState()
		if settings.ClickPetCard then
			rematch:HidePetCard()
		end
	end
	rematch.BottomPanel:Update()
end

-- this makes the border/titlebar/lock appear or disappear (alpha)
function card:UpdateLockState()
	local locked = card.locked
	card:SetAlpha(card.locked and 1 or 0)
	card.Title.Icon:EnableMouse(card.locked)
	card.Title.Type:EnableMouse(card.locked)
	card:EnableMouse(card.locked)
	card.CloseButton:EnableMouse(card.locked)
	card.PinButton:EnableMouse(card.locked)
	for _,button in pairs(card.statButtons) do
		button:EnableMouse(locked)
	end
	for _,button in pairs(card.Front.Bottom.Abilities) do
		button:EnableMouse(locked)
	end
	card.PinButton:SetShown(RematchSettings.PetCardXPos and settings.FixedPetCard and true)
end

function card:OnMouseDown()
	card:StartMoving()
	settings.PetCardXPos, settings.PetCardYPos = card:GetCenter()
	card:UpdateLockState()
end

function card:OnMouseUp()
	card:StopMovingOrSizing()
	settings.PetCardXPos, settings.PetCardYPos = card:GetCenter()
	card:UpdateLockState()
end

function card:PinOnClick()
	settings.PetCardXPos = nil
	settings.PetCardYPos = nil
	rematch:ShowPetCard(card.parent,card.petID,true)
	card:UpdateLockState()
end

function card:OnKeyDown(key)
	if key==GetBindingKey("TOGGLEGAMEMENU") and card.locked then
		rematch:HidePetCard()
		self:SetPropagateKeyboardInput(false)
	else
		self:SetPropagateKeyboardInput(true)
	end
end

function card:TitleButtonOnEnter()
	if card.petID~=0 then
		card.Front:Hide()
		card.Back:Show()
	end
end

function card:TitleButtonOnLeave()
	if card.petID~=0 then
		card.Front:Show()
		card.Back:Hide()
	end
end

--[[ Ability Card ]]

-- instead of duplicating all the tables and parsing to create our own ability tooltip,
-- we just set a default tooltip, copy its strings and textures to our own, then
-- hide the default before it's rendered on screen.

function rematch:ShowAbilityCard(parent,petID,abilityID)

	if rematch:UIJustChanged() then return end

	local tooltip = FloatingPetBattleAbilityTooltip
	if petID and abilityID then
		local _,maxHealth,power,speed
		if type(petID)=="string" then
			_,maxHealth,power,speed = C_PetJournal.GetPetStats(petID)
			abilityCard.TitleBG:SetDesaturated(false)
			abilityCard.Hints.HintsBG:SetDesaturated(false)
		else
			maxHealth,power,speed = 100,0,0 -- missing pets are weak!
			abilityCard.TitleBG:SetDesaturated(true)
			abilityCard.Hints.HintsBG:SetDesaturated(true)
		end

		local _,name,icon,_,_,_,_,noStrongWeakHints = C_PetBattles.GetAbilityInfoByID(abilityID)

		-- do stuff we can handle on our own
		abilityCard.Icon:SetTexture(icon)
		abilityCard.Name:SetText(name)
		abilityCard.Hints:SetShown(not noStrongWeakHints)

		-- now make default do all the work
		FloatingPetBattleAbility_Show(abilityID,maxHealth,power,speed)

		-- at this point, default tooltip is filled in. now copy default's hard work
		abilityCard.Type:SetTexture(tooltip.AbilityPetType:GetTexture())

		local bottom -- tracks the bottom-most line shown, nil if we anchor to the top

		local hasDuration = tooltip.Duration:IsVisible()
		if hasDuration then
			abilityCard.Duration:SetText(tooltip.Duration:GetText())
			bottom = abilityCard.Duration
		end
		abilityCard.Duration:SetShown(hasDuration)

		local hasCooldown = tooltip.MaxCooldown:IsVisible()
		if hasCooldown then
			abilityCard.Cooldown:SetText(tooltip.MaxCooldown:GetText())
			if not bottom then
				abilityCard.Cooldown:SetPoint("TOPLEFT",abilityCard.Duration,"TOPLEFT")
			else
				abilityCard.Cooldown:SetPoint("TOPLEFT",bottom,"BOTTOMLEFT",0,-5)
			end
			bottom = abilityCard.Cooldown
		end
		abilityCard.Cooldown:SetShown(hasCooldown)

		abilityCard.Description:SetText(tooltip.Description:GetText())
		if not bottom then
			abilityCard.Description:SetPoint("TOPLEFT",abilityCard.Duration,"TOPLEFT")
		else
			abilityCard.Description:SetPoint("TOPLEFT",bottom,"BOTTOMLEFT",0,-5)
		end

		if not noStrongWeakHints then -- pet has hints
			abilityCard.Hints.StrongType:SetTexture(tooltip.StrongAgainstType1:GetTexture())
			abilityCard.Hints.WeakType:SetTexture(tooltip.WeakAgainstType1:GetTexture())
			abilityCard.MiddleBG:SetPoint("BOTTOMRIGHT",abilityCard.Hints,"TOPRIGHT",-3,0)
		else
			abilityCard.MiddleBG:SetPoint("BOTTOMRIGHT",-3,3)
		end

		tooltip:Hide() -- we're done with default tooltip, no need to show it

		rematch:SmartAnchor(abilityCard,parent)

		abilityCard:SetAlpha(0) -- going to wait a frame to resize
		abilityCard:SetScript("OnUpdate",card.ResizeAbilityCard)
		rematch:AdjustScale(abilityCard)
		abilityCard:Show()
	end

end

-- let ability card render for a frame to set boundries relative to positions
function card:ResizeAbilityCard()
	self:SetScript("OnUpdate",nil)
	local top = self:GetTop()
	local bottom = self.Description:GetBottom()
	if self.Hints:IsVisible() then
		self:SetHeight(top-bottom+50)
	else
		self:SetHeight(top-bottom+11)
	end
	self:SetAlpha(1)
end

function rematch:HideAbilityCard()
	abilityCard:Hide()
end

-- handles the linking of abilities
function rematch:ChatLinkAbility()
	if IsModifiedClick("CHATLINK") then
		local abilityID = self.abilityID
		local petID = self:GetParent().petID
		if not petID and card:IsVisible() then
			petID = card.petID
		end
		local maxHealth,power,speed,_ = 100,0,0
		if rematch:GetIDType(petID)=="pet" then
			_,maxHealth,power,speed = C_PetJournal.GetPetStats(petID)
		end
		ChatEdit_InsertLink(GetBattlePetAbilityHyperlink(abilityID,maxHealth,power,speed))
		return true
	end
end

function card:PossibleBreedsOnEnter()
	local middle = card.Front.Middle
	if not middle.PossibleBreeds:IsVisible() then
		return
	end
	local btable = middle.BreedTable
	middle.PossibleBreedsHighlight:Show()
	btable:SetFrameLevel(self:GetFrameLevel()+5)
	btable.Footnote:SetText(format(L["All breed data pulled from %s%s\124r."],rematch.hexWhite,GetAddOnMetadata(rematch.breedSource,"Title") or rematch.breedSource))
	btable.Title:SetText(rematch.breedSource=="PetTracker_Breeds" and L["Possible Breeds"] or L["Stats At Level 25 \124cff0070ddRare"])

	btable.Rows = btable.Rows or {}
	btable.Highlight:Hide()
	for _,row in ipairs(btable.Rows) do
		row:Hide() -- clean up all rows
	end

	local speciesID, owned
	if type(card.petID)=="string" then
		speciesID = C_PetJournal.GetPetInfoByPetID(card.petID)
		owned = true
	else
		speciesID = card.petID
	end

	card:FillBreedTable(speciesID,rematch.info) -- gather possible breed data into rematch.info

	for index,info in ipairs(rematch.info) do
		if not btable.Rows[index] then
			btable.Rows[index] = CreateFrame("Frame",nil,btable,"RematchBreedTableTemplate")
			btable.Rows[index]:SetPoint("TOPLEFT",8,-50-(index-1)*16)
		end
		local row = btable.Rows[index]
		row.Breed:SetText(info[1])
		row.Health:SetText(info[2])
		row.Power:SetText(info[3])
		row.Speed:SetText(info[4])
		if owned and info[1]==rematch:GetBreed(card.petID) then
			btable.Highlight:SetPoint("TOPLEFT",row,2,0)
			btable.Highlight:SetPoint("BOTTOMRIGHT",row,-2,-1)
			btable.Highlight:Show()
		end
		row:Show()
	end

	local numBreeds = #rematch.info
	if numBreeds==0 then
		btable.NoBreeds:SetText(L["No known breeds :("])
		btable.NoBreeds:Show()
		btable:SetHeight(32+87)
	else
		btable.NoBreeds:Hide()
		btable:SetHeight(numBreeds*16+87)
	end
	btable:Show()
end

function card:PossibleBreedsOnLeave()
	card.Front.Middle.PossibleBreedsHighlight:Hide()
	card.Front.Middle.BreedTable:Hide()
end

-- takes a table (breeds) and fills it with all known breeds and their stats as a 25 rare: { breedName, health, power, speed }
function card:FillBreedTable(speciesID,breeds)
	wipe(breeds)
	if rematch.breedSource=="BattlePetBreedID" then
		if not BPBID_Arrays.BreedsPerSpecies then
			BPBID_Arrays.InitializeArrays()
		end
		local data,numBreeds = BPBID_Arrays
		if data.BreedsPerSpecies[speciesID] then
			for i=1,#data.BreedsPerSpecies[speciesID] do
				local breed = data.BreedsPerSpecies[speciesID][i]
				local breedText = rematch.breedNames[breed-2] -- rematch:GetBPBIDBreedName(breed)
				local health = ceil((data.BasePetStats[speciesID][1] + data.BreedStats[breed][1]) * 25 * ((data.RealRarityValues[4] - 0.5) * 2 + 1) * 5 + 100 - 0.5)
				local power = ceil((data.BasePetStats[speciesID][2] + data.BreedStats[breed][2]) * 25 * ((data.RealRarityValues[4] - 0.5) * 2 + 1) - 0.5)
				local speed = ceil((data.BasePetStats[speciesID][3] + data.BreedStats[breed][3]) * 25 * ((data.RealRarityValues[4] - 0.5) * 2 + 1) - 0.5)
				tinsert(breeds,{breedText,health,power,speed})
			end
		end
	elseif rematch.breedSource=="LibPetBreedInfo-1.0" then
		local lib = rematch.breedLib
		local data = lib:GetAvailableBreeds(speciesID)
		if data then
			for _,breed in pairs(data) do
				tinsert(breeds,{lib:GetBreedName(breed),lib:GetPetPredictedStats(speciesID,breed,4,25)})
			end
		end
	elseif rematch.breedSource=="PetTracker_Breeds" then
		if PetTracker.Breeds[speciesID] then
			for _,breed in pairs(PetTracker.Breeds[speciesID]) do
				local health, power, speed = unpack(PetTracker.BreedStats[breed])
				health = health*50
				power = power*50
				speed = speed*50
				tinsert(breeds,{(PetTracker:GetBreedIcon(breed,.95)),health>0 and format("%d%%",health) or "-    ",power>0 and format("%d%%",power) or "-    ",speed>0 and format("%d%%",speed) or "-    "})
			end
		end
	end
end

-- when an ability on the pet card is double clicked, search for the ability in the pet panel
function card:SearchAbility(abilityID)
	if abilityID then
		local _,name = C_PetBattles.GetAbilityInfoByID(abilityID)
		if name then
			roster:ClearAllFilters()
			rematch:ShowPets()
			rematch.PetPanel.Top.SearchBox:SetFocus(true)
			rematch.PetPanel.Top.SearchBox:SetText(name)
			rematch.PetPanel.Top.SearchBox:ClearFocus()
		end
	end
end
