local _,L = ...
local rematch = Rematch
local panel = RematchLoadedTeamPanel
local settings

rematch:InitModule(function()
	rematch.LoadedTeamPanel = panel
	settings = RematchSettings
	panel:RegisterForClicks("AnyUp")
	panel:RegisterForDrag("LeftButton")
end)

-- reparents and positions this panel to parent
function panel:Reparent(parent)
	panel:SetParent(parent)
	-- this panel always spans whole bottom of parent frame or journal
	panel:ClearAllPoints()
	if parent==rematch.MiniPanel then
		panel:SetPoint("TOPLEFT",rematch.Frame,"TOPLEFT",4,-24)
		panel.maxWidth = 260
	elseif parent==rematch.LoadoutPanel then
		panel:SetPoint("TOPLEFT",rematch.LoadoutPanel,"TOPLEFT",0,-90)
		panel.maxWidth = 280
	else -- for all of the standalone panels
		if settings.NarrowMode then
			panel:SetPoint("TOPLEFT",rematch.Frame,"TOPLEFT",4,-24)
		else
			panel:SetPoint("BOTTOMLEFT",parent,"TOPLEFT",0,96)
		end
		-- if parenting to a panel in narrow mode, it's the wide 337; otherwise 280 above queue
		panel.maxWidth = settings.NarrowMode and 337 or 280
	end
	panel:Show()
end

function panel:Update()
	local key = settings.loadedTeam
	if key then
		panel.Name:SetText(rematch:GetTeamTitle(settings.loadedTeam,true))
	else
		panel.Name:SetText(BATTLE_PET_SLOTS)
	end
	panel.key = key
	panel.Footnotes.key = key
	local team = RematchSaved[key]

	panel.Favorite:SetShown(team and team.favorite)

	local fx = 4
	local footnotes = panel.Footnotes

	if rematch:HasPreferences(team) then
		footnotes.Preferences:SetPoint("LEFT",fx,0)
		footnotes.Preferences:Show()
		fx = fx + 21
	else
		footnotes.Preferences:Hide()
	end
	if team and team.notes then
		footnotes.Notes:SetPoint("LEFT",fx,0)
		footnotes.Notes:Show()
		fx = fx + 21
	else
		footnotes.Notes:Hide()
	end
	if rematch.Frame:IsVisible() and settings.Minimized and settings.MiniMinimized then
		footnotes.Maximize:SetPoint("LEFT",fx-5,0)
		footnotes.Maximize:Show()
		fx = fx + 21
		footnotes.Close:SetPoint("LEFT",fx-5,0)
		footnotes.Close:Show()
		fx = fx + 21
	else
		footnotes.Maximize:Hide()
		footnotes.Close:Hide()
	end

	local footnoteWidth = fx + 4

	local panelWidth = panel.maxWidth or 280

	if footnoteWidth>10 then
		panel.Footnotes:SetWidth(footnoteWidth)
		panel.Footnotes:Show()
		panel:SetWidth(panelWidth-footnoteWidth-3)
	else
		panel.Footnotes:Hide()
		panel:SetWidth(panelWidth)
	end
end

function panel:OnClick(button)
	if button=="RightButton" and self.key then
		rematch:SetMenuSubject(self.key)
		rematch:ShowMenu("TeamMenu","cursor")
	elseif self.key then
		rematch:LoadTeam(self.key)
	end
end

function panel:OnDragStart()
	if rematch.Frame:IsVisible() then
		Rematch.Frame:MoveStart()
	end
end

function panel:OnDragStop()
	Rematch.Frame:MoveStop()
	self:GetScript("OnMouseUp")(self)
end

function panel:OnEnter()
	local body = ""
	if settings.loadedTeam then
		local teamName = rematch:GetTeamTitle(settings.loadedTeam)
		if self.Name:IsTruncated() then
			body = format("%s%s\n",body,rematch:GetTeamTitle(settings.loadedTeam,true))
		end
		if type(settings.loadedTeam)=="number" then
			local npcName = rematch:GetNameFromNpcID(settings.loadedTeam)
			if title~=npcName then
				body = format("%s%s%s\124r\n",body,rematch.hexGrey,npcName)
			end
		end
		body = format("%s%s %s\n%s %s",body,rematch.LMB,L["Reload Team"],rematch.RMB,L["Team Options"])
	else
		body = L["When a team is loaded its name goes here. You can reload the team by clicking here or right-click for team options such as setting notes."]
	end
	rematch.ShowTooltip(self,L["Last Loaded Team"],body)
end
